@@ -21,7 +21,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: ike-scan.c,v 1.58 2003/05/10 16:09:03 rsh Exp $
+ * $Id: ike-scan.c,v 1.67 2003/07/31 15:11:43 rsh Exp $
  *
  * ike-scan -- The IKE Scanner
  *
@@ -46,8 +46,7 @@
 
 #include "ike-scan.h"
 
-#define MAX_PAYLOAD 13	/* Maximum defined payload number */
-static char rcsid[] = "$Id: ike-scan.c,v 1.58 2003/05/10 16:09:03 rsh Exp $";   /* RCS ID for ident(1) */
+static char rcsid[] = "$Id: ike-scan.c,v 1.67 2003/07/31 15:11:43 rsh Exp $";   /* RCS ID for ident(1) */
 
 /* Global variables */
 struct host_entry *rrlist = NULL;	/* Round-robin linked list "the list" */
@@ -75,7 +74,6 @@ int id_data_len;
 uint32_t kx_data[48];		/* Key exchange data.  48 is largest needed */
 int kx_data_len;		/* Key exchange data len in 32bit longwords */
 int verbose=0;
-char vendor_id[MAXLINE];		/* Vendor ID string */
 int vendor_id_flag = 0;			/* Indicates if VID to be used */
 char trans_str[MAXLINE];		/* Custom transform string */
 int trans_flag = 0;			/* Indicates custom transform */
@@ -101,7 +99,8 @@ struct transform
 };
 struct transform trans[8];		/* Transform payload */
 struct isakmp_vid vid_hdr;		/* Vendor ID header */
-md5_byte_t vid_md5[16];			/* Vendor ID data - md5 digest */
+unsigned char *vid_data;		/* Binary Vendor ID data */
+int vid_data_len;			/* Vendor ID data length */
 
 const char *auth_methods[] = { /* Authentication methods from RFC 2409 Appendix A */
    "UNSPECIFIED",		/* 0 */
@@ -209,6 +208,7 @@ main(int argc, char *argv[]) {
    unsigned long end_timediff=0; /* Time since last packet received in ms */
    int arg_str_space;		/* Used to avoid buffer overruns when copying */
    char patfile[MAXLINE];	/* IKE Backoff pattern file name */
+   unsigned char *cp;
 /*
  *	Open syslog channel and log arguments if required.
  *	We must be careful here to avoid overflowing the arg_str buffer
@@ -239,7 +239,6 @@ main(int argc, char *argv[]) {
  */
    while ((arg=getopt_long_only(argc, argv, short_options, long_options, &options_index)) != -1) {
       switch (arg) {
-         md5_state_t context;
          int i;
          case 'f':	/* --file */
             strncpy(filename, optarg, MAXLINE);
@@ -291,15 +290,16 @@ main(int argc, char *argv[]) {
             exit(0);
             break;
          case 'e':	/* --vendor */
-            strncpy(vendor_id, optarg, MAXLINE);
+            if (strlen(optarg) % 2) {	/* Length is odd */
+               err_msg("Length of --vendor argument must be even (multiple of 2).");
+            }
             vendor_id_flag=1;
-            md5_init(&context);
-            md5_append(&context, (const md5_byte_t *)vendor_id, strlen(vendor_id));
-            md5_finish(&context, vid_md5);
-            printf("vid_md5: ");
-            for (i=0; i<16; i++)
-               printf("%.2x",vid_md5[i]);
-            printf("\n");
+            vid_data_len=strlen(optarg)/2;
+            if ((vid_data = malloc(vid_data_len)) == NULL)
+               err_sys("malloc");
+            cp = vid_data;
+            for (i=0; i<vid_data_len; i++)
+               *cp++=hstr_i(&optarg[i*2]);
             break;
          case 'a':	/* --trans */
             strncpy(trans_str, optarg, MAXLINE);
@@ -454,6 +454,10 @@ main(int argc, char *argv[]) {
       err_sys("gettimeofday");
    }
    initialise_ike_packet();
+/*
+ *	Check ISAKMP structure sizes.
+ */
+   check_struct_sizes();
 /*
  *	Display initial message.
  */
@@ -713,6 +717,8 @@ display_packet(int n, char *packet_in, struct host_entry *he, struct in_addr *re
    struct isakmp_sa sa_hdr_in;
    struct isakmp_proposal sa_prop_in;
    struct isakmp_notification notification_in;
+   struct isakmp_vid vid_hdr_in;
+   int vid_data_len_in;
    int msg_len;                 /* Size of notification message in bytes */
    int msg_type;                /* Notification message type */
    char msg_in[MAXLINE];        /* Notification message */
@@ -727,6 +733,8 @@ display_packet(int n, char *packet_in, struct host_entry *he, struct in_addr *re
    if ((he->addr).s_addr != recv_addr->s_addr)
       cp += sprintf(cp, "(%s) ", inet_ntoa(*recv_addr));
    *cp = '\0';
+
+   cp = packet_in;	/* Save original start of packet. Shouldn't need this */
 /*
  *	Check that the received packet is at least as big as the ISAKMP
  *	header before we try to copy it into an ISAKMP header struct.
@@ -770,7 +778,32 @@ display_packet(int n, char *packet_in, struct host_entry *he, struct in_addr *re
             sprintf(xchg_type, "UNKNOWN Mode (%u)", hdr_in.isa_xchg);
          }
          decode_transform(packet_in, n, sa_prop_in.isap_notrans);
-         printf("%sIKE %s Handshake returned (%d transforms)\n", ip_str, xchg_type, sa_prop_in.isap_notrans);
+         if (sa_prop_in.isap_notrans == 1) {
+            printf("%sIKE %s Handshake returned.",
+                   ip_str, xchg_type);
+         } else {	/* More than 1 transform - shouldn't happen */
+            printf("%sIKE %s Handshake returned (%d transforms).",
+                   ip_str, xchg_type, sa_prop_in.isap_notrans);
+         }
+/*
+ *	If the payload after SA is VID, print the associated data as hex.
+ *	We should really check the packet size before copying rather than
+ *	just trusting the length fields.
+ */
+         if (sa_hdr_in.isasa_np == ISAKMP_NEXT_VID) {
+            int i;
+
+            cp += sizeof(hdr_in) + ntohs(sa_hdr_in.isasa_length);
+            memcpy(&vid_hdr_in, cp, sizeof(vid_hdr_in));
+            cp += sizeof(vid_hdr_in);	/* cp now points at VID data */
+            vid_data_len_in=ntohs(vid_hdr_in.isavid_length) - sizeof(vid_hdr_in);
+            printf(" VID Data=");
+            for (i=0; i<vid_data_len_in; i++) {
+               printf("%.2x", (unsigned char) *cp);
+               cp++;
+            }
+         }
+         printf("\n");
       } else {
          printf("%sIKE Handshake returned (%d byte packet too short to decode)\n", ip_str, n);
       }
@@ -821,6 +854,11 @@ display_packet(int n, char *packet_in, struct host_entry *he, struct in_addr *re
 
 /*
  *	decode-transform -- Decode an IKE transform payload
+ *
+ *	On entry:
+ *	packet_in points to the start of the transform payload
+ *	n is the total packet size
+ *	ntrans is the number of transforms (almost always 1)
  */
 void
 decode_transform(char *packet_in, int n, int ntrans) {
@@ -877,9 +915,9 @@ send_packet(int s, struct host_entry *he) {
    if (vendor_id_flag) {
       memcpy(cp, &vid_hdr, sizeof(vid_hdr));
       cp += sizeof(vid_hdr);
-      memcpy(cp, &vid_md5, sizeof(vid_md5));
-      cp += sizeof(vid_md5);
-      buflen += sizeof(vid_hdr)+sizeof(vid_md5);
+      memcpy(cp, vid_data, vid_data_len);
+      cp += vid_data_len;
+      buflen += sizeof(vid_hdr)+vid_data_len;
    } 
 /*
  *	Update the last send times for this host.
@@ -993,7 +1031,7 @@ initialise_ike_packet(void) {
    hdr.isa_msgid = 0;                   /* MBZ for phase-1 */
    len=sizeof(hdr)+sizeof(sa_hdr)+sizeof(sa_prop);
    if (vendor_id_flag) {
-      len += (sizeof(vid_hdr) + sizeof(vid_md5));
+      len += sizeof(vid_hdr) + vid_data_len;
    }
    if (trans_flag) {
       len += sizeof(trans[0]);
@@ -1203,7 +1241,7 @@ initialise_ike_packet(void) {
  *	Vendor ID Payload (Optional)
  */
    vid_hdr.isavid_np = ISAKMP_NEXT_NONE;	/* No Next payload */
-   vid_hdr.isavid_length = htons(sizeof(vid_hdr) + sizeof(vid_md5));	/* Length of MD5 digest */
+   vid_hdr.isavid_length = htons(sizeof(vid_hdr) + vid_data_len);	/* Length of VID data */
 }
 
 /*
@@ -1226,11 +1264,15 @@ dump_list(void) {
 
 /*
  *	dump_backoff -- Display contents of backoff list for debugging
+ *
+ *	This displays the contents of the backoff pattern list.  It is useful
+ *	when debugging to check that the patterns have been loaded correctly
+ *	from the backoff patterns file.
  */
 void
 dump_backoff(void) {
    struct pattern_list *pl;
-   struct time_list *pp;
+   struct pattern_entry_list *pp;
    int i;
 
    printf("Backoff Pattern List:\n\n");
@@ -1241,11 +1283,33 @@ dump_backoff(void) {
       printf("%d\t%s\t%d\t", i, pl->name, pl->num_times);
       pp = pl->recv_times;
       while (pp != NULL) {
-         printf("%ld.%.6ld, ", (long)pp->time.tv_sec, (long)pp->time.tv_usec);
+/*
+ *  Only print the fractional seconds part if required (generally it's not).
+ *  We cast to long because some OSes define tv_sec/tv_usec as long and
+ *  others define them as long.
+ */
+         if (pp->time.tv_usec) {
+            printf("%ld.%.6ld", (long)pp->time.tv_sec, (long)pp->time.tv_usec);
+         } else {
+            printf("%ld", (long)pp->time.tv_sec);
+         }
+/*
+ * Display the fuzz value for this pattern entry if it is not the default.
+ */
+         if (pp->fuzz != pattern_fuzz)
+            printf("/%d", pp->fuzz);
+/*
+ * Print a newline if we're at the end of this pattern, otherwise print a
+ * comma and space before the next time element.
+ */
          pp = pp->next;
+         if (pp == NULL) {
+            printf("\n");
+         } else {
+            printf(", ");
+         } 
       }
       pl = pl->next;
-      printf("\n");
       i++;
    } /* End While */
    printf("\nTotal of %d backoff pattern entries.\n\n", i-1);
@@ -1329,7 +1393,7 @@ match_pattern(struct host_entry *he) {
    while (pl != NULL) {
       if (he->num_recv == pl->num_times && pl->recv_times != NULL) {
          struct time_list *hp;
-         struct time_list *pp;
+         struct pattern_entry_list *pp;
          struct timeval diff;
          struct timeval prev_time;
          int match;
@@ -1344,7 +1408,7 @@ match_pattern(struct host_entry *he) {
          while (pp != NULL && hp != NULL) {
             if (i > 1)
                timeval_diff(&(hp->time), &prev_time, &diff);
-            if (!times_close_enough(&(pp->time), &diff)) {
+            if (!times_close_enough(&(pp->time), &diff, pp->fuzz)) {
                match = 0;
                break;
             }
@@ -1369,13 +1433,13 @@ match_pattern(struct host_entry *he) {
  *	                      of each other.  Otherwise return 0.
  */
 int
-times_close_enough(struct timeval *t1, struct timeval *t2) {
+times_close_enough(struct timeval *t1, struct timeval *t2, unsigned fuzz) {
 struct timeval diff;
 int diff_ms;
 
    timeval_diff(t1, t2, &diff);	/* diff = t1 - t2 */
    diff_ms = abs(1000*diff.tv_sec + diff.tv_usec/1000);
-   if (diff_ms <= pattern_fuzz) {
+   if (diff_ms <= fuzz) {
       return 1;
    } else {
       return 0;
@@ -1430,11 +1494,15 @@ add_pattern(char *line) {
    int tabseen;
    struct pattern_list *pe;	/* Pattern entry */
    struct pattern_list *p;	/* Temp pointer */
-   struct time_list *te;
-   struct time_list *tp;
+   struct pattern_entry_list *te;
+   struct pattern_entry_list *tp;
    char *endp;
    int i;
-   double back;
+   long back_sec;
+   long back_usec;
+   char back_usec_str[7];       /* Backoff microseconds as string */
+   int len;
+   unsigned fuzz;	/* Pattern matching fuzz in ms */
 /*
  *	Allocate new pattern list entry and add to tail of patlist.
  */
@@ -1483,12 +1551,54 @@ add_pattern(char *line) {
    i=0;
    endp=pat;
    while (*endp != '\0') {
-      back=strtod(endp, &endp);
-      if ((te = malloc(sizeof(struct time_list))) == NULL)
+/*
+ *      Convert the integer seconds part of the backoff pattern entry.
+ */
+      back_sec=strtol(endp, &endp, 10);
+/*
+ *      If there is a "." after the integer part, then there are fractional
+ *      seconds to convert.  Convert the fractional seconds into a string
+ *      representation of milliseconds by zero-extending to 6 digits and
+ *      then convert to long.
+ */
+      if (*endp == '.') {
+         endp++;
+         len=0;
+         for (len=0; len<6; len++){
+            if (isdigit(*endp)) {
+               back_usec_str[len] = *endp;
+               endp++;
+            } else {
+               back_usec_str[len] = '0';
+            }
+         }
+         while (isdigit(*endp))
+            endp++;	/* Skip any fractional digits past 6th */
+         back_usec_str[len] = '\0';
+         back_usec=strtol(back_usec_str, NULL, 10);
+      } else {	/* No fractional seconds part */
+         back_usec=0;
+      }
+/*
+ *      If there is a "/" after the number, this represents a fuzz value
+ *      in milliseconds.
+ */
+      if (*endp == '/') {
+         endp++;
+         fuzz=strtol(endp, &endp, 10);
+      } else {
+         fuzz = pattern_fuzz;
+      }
+/*
+ *      Allocate and fill in new pattern_entry_list structure for this backoff
+ *      pattern entry and add it onto the tail of this pattern entry.
+ */
+      if ((te = malloc(sizeof(struct pattern_entry_list))) == NULL)
          err_sys("malloc");
       te->next=NULL;
-      te->time.tv_sec = floor(back);
-      te->time.tv_usec = (back - te->time.tv_sec) * 1000000;
+      te->time.tv_sec = back_sec;
+      te->time.tv_usec = back_usec;
+      te->fuzz = fuzz;
       tp = pe->recv_times;
       if (tp == NULL) {
          pe->recv_times = te;
@@ -1497,13 +1607,78 @@ add_pattern(char *line) {
             tp = tp->next;
          tp->next = te;
       }
+/*
+ *	Move on to next pattern entry.
+ */
       if (*endp == ',')
          endp++;
       i++;
-   }
+   }	/* End While */
    pe->num_times=i;
 }
 
+/*
+ *	Check that the sizes of the various structs are what we expect them
+ *	to be.  Issue a warning message if they are not.
+ *
+ *	There are several places in the ike-scan code where we copy structs
+ *	to character arrays and vice versa.  E.g. send_packet().
+ *
+ *	Although this is not condoned in C in practice it is OK in practice
+ *	providing that the sizes of the fields is correct and there is no
+ *	padding between fields (e.g. for alignment purposes).  This function
+ *	checks for both of these problems.
+ *
+ *	All of the potential problems that this functions checks for occur at
+ *	compile time, so any given ike-scan binary will always behave in the
+ *	same way.  Perhaps this function would be better written as a
+ *	"make test" check.  However, as it's just a single comparison, it
+ *	doesn't add any significant overhead to run it every time.
+ */
+void
+check_struct_sizes() {
+   int actual_total;
+
+   actual_total = sizeof(struct isakmp_hdr) +
+                  sizeof(struct isakmp_sa) +
+                  sizeof(struct isakmp_proposal) +
+                  sizeof(struct transform) +
+                  sizeof(struct isakmp_vid) +
+                  sizeof(struct isakmp_notification);
+
+   if (actual_total != EXPECTED_TOTAL) {
+      fprintf(stderr, "WARNING: Total size of ISAKMP structures is %d bytes, expected %d bytes\n", actual_total, EXPECTED_TOTAL);
+      fprintf(stderr, "\tThis will probably cause ike-scan to fail because the IKE packet\n");
+      fprintf(stderr, "\tstructure will not be correct.\n");
+      fprintf(stderr, "\tThis problem can be caused by incorrect type sizes for the various\n");
+      fprintf(stderr, "\ttypes used in the ISAKMP structures (e.g. u_int_8, u_int_16\n");
+      fprintf(stderr, "\tand u_int_32) or by alignment padding.\n");
+   }
+}
+
+/*
+ *	Convert a two-digit hex string with to unsigned int.
+ *	E.g. "0A" would return 10.
+ *	Note that this function does no sanity checking, it's up to the
+ *	caller to ensure that *cptr points to at least two hex digits.
+ *	This function is a modified version of hstr_i at www.snippets.org.
+ */
+unsigned int hstr_i(char *cptr)
+{
+      unsigned int i;
+      unsigned int j = 0;
+      int k;
+
+      for (k=0; k<2; k++) {
+            i = *cptr++ - '0';
+            if (9 < i)
+                  i -= 7;
+            j <<= 4;
+            j |= (i & 0x0f);
+      }
+      return(j);
+}
+
 /*
  *	usage -- display usage message and exit
  */
@@ -1575,9 +1750,9 @@ usage(void) {
    fprintf(stderr, "\t\t\timplementations may require different values.\n");
    fprintf(stderr, "\n--auth=<n> or -m <n>\tSet auth. method to <n>, default=%d (%s).\n", DEFAULT_AUTH_METHOD, auth_methods[DEFAULT_AUTH_METHOD]);
    fprintf(stderr, "\t\t\tRFC defined values are 1 to 5.  See RFC 2409 Appendix A.\n");
+   fprintf(stderr, "\t\t\tCheckpoint hybrid mode is 64221.\n");
    fprintf(stderr, "\n--version or -V\t\tDisplay program version and exit.\n");
-   fprintf(stderr, "\n--vendor=<v> or -e <v>\tSet vendor id string to MD5 hash of <v>.\n");
-   fprintf(stderr, "\t\t\tNote: this is currently experimental.\n");
+   fprintf(stderr, "\n--vendor=<v> or -e <v>\tSet vendor id string to hex value <v>.\n");
    fprintf(stderr, "\n--trans=<t> or -a <t>\tUse custom transform <t> instead of default set.\n");
    fprintf(stderr, "\t\t\t<t> is specified as enc,hash,auth,group. e.g. 2,3,1,5.\n");
    fprintf(stderr, "\t\t\tSee RFC 2409 Appendix A for details of which values\n");
@@ -1601,6 +1776,8 @@ usage(void) {
    fprintf(stderr, "\t\t\tthe backoff patterns file.  Larger values allow for\n");
    fprintf(stderr, "\t\t\thigher variance but also increase the risk of\n");
    fprintf(stderr, "\t\t\tfalse positive identifications.\n");
+   fprintf(stderr, "\t\t\tAny per-pattern-entry fuzz specifications in the\n");
+   fprintf(stderr, "\t\t\tpatterns file will override the value set here.\n");
 #ifdef __CYGWIN__
    fprintf(stderr, "\n--patterns=<f> or -p <f> Use IKE patterns file <f>,\n");
    fprintf(stderr, "\t\t\tdefault=%s in ike-scan.exe dir.\n", PATTERNS_FILE);
