@@ -21,7 +21,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: ike-scan.h,v 1.36 2003/05/09 15:49:17 rsh Exp $
+ * $Id: ike-scan.h,v 1.40 2003/07/16 11:26:16 rsh Exp $
  *
  * ike-scan.h -- Header file for IKE Scanner (ike-scan)
  *
@@ -41,7 +41,6 @@
 #include <ctype.h>
 #include <stdarg.h>
 #include <errno.h>
-#include <math.h>
 #else
 #error This program requires the ANSI C Headers
 #endif
@@ -121,6 +120,8 @@
 #define SYSLOG 1			/* Use syslog if defined */
 #define SYSLOG_FACILITY LOG_USER	/* Syslog facility to use */
 #define PATTERNS_FILE "ike-backoff-patterns" /* Backoff patterns filename */
+#define MAX_PAYLOAD 13			/* Maximum defined payload number */
+#define EXPECTED_TOTAL 100		/* Expected ISAKMP header size total */
 
 /* Structures */
 struct host_entry {
@@ -142,10 +143,16 @@ struct time_list {
    struct time_list *next;
 };
 
+struct pattern_entry_list {
+   struct timeval time;
+   unsigned fuzz;
+   struct pattern_entry_list *next;
+};
+
 struct pattern_list {
    char *name;
    int num_times;
-   struct time_list *recv_times;
+   struct pattern_entry_list *recv_times;
    struct pattern_list *next;
 };
 
@@ -174,5 +181,7 @@ void dump_times(void);
 void add_recv_time(struct host_entry *);
 void add_pattern(char *);
 char *match_pattern(struct host_entry *);
-int times_close_enough(struct timeval *, struct timeval *);
+int times_close_enough(struct timeval *, struct timeval *, unsigned);
 void dump_backoff(void);
+void check_struct_sizes(void);
+unsigned int hstr_i(char *);
